local level = require("level")

local images = {}
images.debug_atlas = "assets/gfx/debug.png"
images.dungeon_atlas = "assets/gfx/0x72_DungeonTilesetII_v1.6.png"
images.powerups = "assets/gfx/powerups.png"
images.dmgdigits = "assets/gfx/dmgdigits.png"
images.cursor = "assets/gfx/cursor.png"
images.panel = "assets/gfx/panel.png"
images.tabs = "assets/gfx/tabs.png"
images.close = "assets/gfx/close.png"
images.keys = "assets/gfx/kb_light_alphanumeric.png"
images.button = "assets/gfx/button.png"
images.shrine = "assets/gfx/shrine.png"
images.title = "assets/gfx/title.png"


local sprites = require("assets/sprites")
local animations = require("assets/animations")


local tilesets = {}
tilesets.floors = {"dungeon_atlas", 16, 16, 16, 64, 48, 48}
tilesets.dmgdigits = {"dmgdigits", 6, 8}

local levels = {}
levels.debuglvl = {"ts", "assets/debuglvl.lua"}

local music = {}
music.waveMusic = {
    source = love.audio.newSource("assets/music/wave-music.mp3", "stream"),
    volume = 1
}

local assets = {}

local function fnmap(t, fn)
    local out = {}
    for k, v in pairs(t) do
        out[k] = fn(v)
    end
    return out
end

local function loadAll()
    assets.images = fnmap(images, function(v)
        return love.graphics.newImage(v)
    end)


    assets.sprites = fnmap(sprites, function(v)
        local atlasName,x,y,w,h,ox,oy = unpack(v)
        local img = assets.images[atlasName]
        return {atlasName, love.graphics.newQuad(x, y, w, h, img:getWidth(), img:getHeight()), ox or 0, oy or 0, w, h}
    end)

    assets.tilesets = fnmap(tilesets, function(v)
        local atlasName, w, h, sx, sy, iw, ih = unpack(v)
        local img = assets.images[atlasName]
        sx = sx or 0
        sy = sy or 0
        ix = iw or img:getWidth()
        iy = ih or img:getHeight()

        
        local qs = {}

        local cols, rows = ix / w, iy / h
        for y = 0, rows - 1 do
            for x = 0, cols - 1 do
                table.insert(qs, love.graphics.newQuad(sx + x * w, sy + y * h, w, h, img:getWidth(), img:getHeight()))
            end
        end
        local ts = {
            atlasName = atlasName,
            tileWidth = w,
            tileHeight = h,
            cols = cols,
            rows = rows,
            quads = qs
        }
        return  ts
    end)

    assets.levels = fnmap(levels, function(v)
        local tilesetId, path = unpack(v)
        local lvl = level.loadMap(path)
        lvl.tileset = assets.tilesets[tilesetId]
        return lvl
    end)

    assets.animations = animations
    assets.music = music
        
end

loadAll()


return assets