local bump = require("lib/bump")
local drawing = require("drawing")

local world

local function defaultFilter(item, other)
    if other.solid then
        if item.projectile then
            return "touch"
        end
        return "slide"
    end
    if item.target and item.target == other.layer then
        return "cross"
    end
end

local collision = {}

function collision.init()
    world = bump.newWorld()
end

function collision.addBody(obj)
    if (obj.cx and obj.cy and obj.cr) then
        world:add(obj, obj.x + obj.cx - obj.cr, obj.y + obj.cy - obj.cr, obj.cr * 2, obj.cr * 2)
    elseif (obj.w and obj.h) then
        world:add(obj, obj.x, obj.y, obj.w, obj.h)
    end
end

function collision.removeBody(obj)
    world:remove(obj)
end

function collision.drawDebug()
    local items, len = world:getItems()

    for i = 1, len do
        local x,y,w,h = world:getRect(items[i])
        drawing.rect("line", math.round(x), math.round(y), w, h)
        drawing.col(0, 1, 1, 1)
        drawing.points(items[i].x, items[i].y)
        drawing.col(1, 1, 1, 1)
    end
    
end

function collision.moveObject(obj, vx, vy, filter)
    filter = filter or defaultFilter
    local goalX, goalY = obj.x + obj.cx - obj.cr + vx, obj.y + obj.cy - obj.cr + vy
    local actualX, actualY, cols, len = world:move(obj, goalX, goalY, filter)
    obj.x, obj.y = actualX -  obj.cx + obj.cr, actualY -  obj.cy + obj.cr

    if len > 0 then
        -- return first colliding object
        table.sort(cols, function(a, b)
            return a.ti <= b.ti
        end)
        return cols[1]
    end
end

function collision.query(x, y, w, h, filter)
    return world:queryRect(x, y, w, h, filter)
end

function collision.raycast(x1, y1, x2, y2, filter)
    return world:querySegmentWithCoords(x1, y1, x2, y2, filter)
end

function collision.bodyOverlaps(body, x, y, w, h)
    if not world:hasItem(body) then
        return
    end

    local rx, ry, rw, rh = world:getRect(body)

    return not (rx + rw < x) and
        not (rx > x + w) and
        not (ry + rh < y) and
        not (ry > y + h)
end

return collision