local assets = require("assets")


local canvas

function updateCanvas(w, h)
    if canvas then
        canvas:release()
    end

    local dwidth = D_WIDTH
    local dheight = D_HEIGHT

    if h > w then
        dwidth = D_HEIGHT
        dheight = D_WIDTH
    end

    local wScale = math.floor(w / dwidth)
    local hScale = math.floor(h / dheight)
    local scale = math.min(wScale, hScale)

    local wDiff = (w / scale) - dwidth
    local hDiff = (h / scale) - dheight
    WIDTH =  math.min(love.graphics.getWidth(), math.floor(dwidth + wDiff))
    HEIGHT = math.min(love.graphics.getHeight(), math.floor(dheight + hDiff))
    canvas = love.graphics.newCanvas(WIDTH, HEIGHT, { dpiscale = 1})
end

updateCanvas(love.graphics.getWidth(), love.graphics.getHeight())

local drawing = {}


function love.resize(w, h)
    updateCanvas(w, h)
end

function drawing.start()
    love.graphics.setCanvas(canvas)
end

function drawing.stop()
    love.graphics.setCanvas()
end

function drawing.saveScreenshot(filename)
    filename = filename or "screenshot.png"
    local img = canvas:newImageData()
    img:encode("png", filename)
    img:release()
end

drawing.cls = love.graphics.clear
drawing.col = love.graphics.setColor
drawing.rect = love.graphics.rectangle
drawing.printf = love.graphics.printf 
drawing.line = love.graphics.line 
drawing.circle = love.graphics.circle
drawing.points = love.graphics.points
drawing.draw = love.graphics.draw
drawing.print = love.graphics.print

function drawing.present()
    love.graphics.setCanvas()
    local scale = math.min(love.graphics.getWidth() / WIDTH, love.graphics.getHeight() / HEIGHT)
    scale = math.floor(scale)
    love.graphics.draw(canvas, 0, 0, 0, scale, scale)
    -- love.graphics.draw(canvas, math.round(love.graphics.getWidth() / 2), math.round(love.graphics.getHeight() / 2), 0, scale, scale, WIDTH / 2, HEIGHT / 2)
end

function drawing.sprite(spriteId, x, y, r, fx, fy)
    if not assets.sprites[spriteId] then
        return
    end
    r = r or 0
    sx = fx and -1 or 1
    sy = fy and -1 or 1
    
    local imgId, quad, ox, oy = unpack(assets.sprites[spriteId])
    local img = assets.images[imgId]

    love.graphics.draw(img, quad, math.round(x), math.round(y), r, sx, sy, ox, oy)
end

function drawing.tile(tilesetName, id, x, y)
    local tileset = assets.tilesets[tilesetName]
    local img = assets.images[tileset.atlasName]
    local quad = tileset.quads[id]

    love.graphics.draw(img, quad, x, y)
end

function drawing.camera(x, y, ox, oy)
    if x or y then
        x = math.round(x) or 0
        y = math.round(y) or 0
        ox = ox or 0 --or WIDTH / 2
        oy = oy or 0 --or HEIGHT / 2
        love.graphics.push()
        love.graphics.translate(-x + ox, -y + oy)
    else
        love.graphics.pop()
    end
end

function drawing.spriteSize(spriteId)
    if not assets.sprites[spriteId] then
        return 0, 0
    end
    local spr = assets.sprites[spriteId]
    return spr[5], spr[6]
end


return drawing