local entity = require("entity")
local collision = require("collision")

local enemydb = {}
-- animation, cx, cy, cr, hp?, more things? 
enemydb.thing = {"swampy_anim", 0, -7, 6, 3}

for k, v in pairs(enemydb) do 
    local animId, cx, cy, cr, hp = unpack(v)
    enemydb[k] = {
        animId = animId,
        cx = cx, 
        cy = cy,
        cr = cr,
        hp = hp,
    }
end

local enemy = {}

function enemy.make(type, x, y) 
    local e = entity.make(enemydb[type], x,y )    
    e.update = enemy.update
    e.layer = "enemies"
    e.state = "idle"
    e.timer = math.min(1, 0.3 + love.math.random())
    return e
end

local function getClosestReactorQuadrant(obj, gameData) 
    local reactor = gameData.reactor
    
    local items, count = collision.raycast(obj.x, obj.y, reactor.x + 16, reactor.y + 16, function(item)
        return item == reactor
    end)
    if count > 0 then
        return items[1].x1, items[1].y1
    end
end

local function getClosestPlayerTarget(obj)
    local items, count = collision.query(-WIDTH, -HEIGHT, 3 * WIDTH, 3 * HEIGHT, function(item)
        return item.layer == "player"
    end)

    table.sort(items, function(l, r) 
        local lDist = math.dist(l.x, l.y, obj.x, obj.y)
        local rDist = math.dist(r.x, r.y, obj.x, obj.y)
        return lDist <= rDist
    end)

    return items[1]
end

function enemy.update(dt, obj, gameData)
    obj.timer = math.max(0, obj.timer - dt)

    if obj.state == "idle" and obj.timer == 0 then
        local targetX, targetY = gameData.reactor.x + 16, gameData.reactor.y +16-- getClosestReactorQuadrant(obj, gameData)
        local player = getClosestPlayerTarget(obj)
        local playerDist = math.dist(obj.x, obj.y, player.x, player.y)
        
        if  playerDist < 60 then
            targetX = player.x
            targetY = player.y
        end
        

        local tpx, tpy = math.norm(targetX - obj.x, targetY - obj.y)
        obj.vx = tpx * 180
        obj.vy = tpy * 180
        obj.state = "moving"
        obj.hasHit = false

    elseif obj.state == "moving" then
        local hit = collision.moveObject(obj, obj.vx * dt, obj.vy * dt, function(item, other)
            if other.layer == "player"   then
                return "cross"
            end
            if other.name == "reactor" then 
                return "touch"
            end
        end)
        if hit and not obj.hasHit then 
            if hit.other.hp then
                hit.other.hp = hit.other.hp - 1
            end
            if hit.other == gameData.player then
                hit.other.state = "hit"
                hit.other.vx = obj.vx / 2
                hit.other.vy = obj.vy / 2
            end

            local dx, dy = math.norm(obj.vx, obj.vy)
            obj.vx = -dx * 60
            obj.vy = -dy * 60
            obj.hasHit = true
        else 

            local friction = 0.9
            obj.vx = obj.vx * friction
            obj.vy = obj.vy * friction 
            if math.len(obj.vx, obj.vy) < 0.05 then
                obj.vx = 0
                obj.vy = 0
                obj.state = "idle"
                obj.timer = love.math.random(20, 60) / 100
            end
        end
    end
end

return enemy