local assets = require("assets")
local drawing = require("drawing")
local collision = require("collision")

local entity = {}

function entity.make(props, x, y)
    local e = {
        x = 0,
        y = 0,
        vx = 0, vy = 0,
        -- cx = 0, cy = 0, cr = 0, 
        animId = nil, animT = 0, animFrame = 1,
        visible = true,
    }
    e.x = x or 0
    e.y = y or 0

    for k,v in pairs(props) do
        e[k] = v
    end

    collision.addBody(e)
    return e
end

local animationFrameTime = 1 / 60

local function updateAnimation(dt, e)
    if not e.animId then
        return
    end
    local frameTime, frames = unpack(assets.animations[e.animId])
    
    e.animT = e.animT + dt
    if e.animT > (frameTime * animationFrameTime) then
        e.animT = 0
        e.animFrame = (e.animFrame % #frames) + 1
        e.sprite = frames[e.animFrame]
    end
end

function entity.playAnimation(e, animId)
    if e.animId ~= animId then
        local frameTime, frames = unpack(assets.animations[animId])

        e.animId = animId
        e.animFrame = 1
        e.animT = 0
        e.sprite = frames[1]
    end
end

function entity.draw(e, ox, oy)
    if e.sprite and e.visible then
        local x = ox or e.x
        local y = oy or e.y
        drawing.sprite(e.sprite, x, y, e.rotation or 0, e.fx or false, e.fy or false)
    end
end

function entity.update(dt, e, gameData)
    if e.update then
        e.update(dt, e, gameData)
    end

    updateAnimation(dt, e)
end

function entity.distance(a, b) 
    local dx, dy = (b.x + b.cx) - (a.x + a.cx), (b.y + b.cy) - (a.y + a.cy)
    return math.sqrt(dx*dx + dy*dy)
end

function entity.overlap(a, b)
    return entity.distance(a, b) <= (a.cr + b.cr)
end

return entity