local Signal = require("lib.hump.signal")

local bindings = {}
bindings.up = {keys={"w", "up"}, buttons={"dpup"}, axis={"lefty", -0.5}}
bindings.down = {keys={"s", "down"}, buttons={"dpdown"}, axis={"lefty", 0.5}}
bindings.left = {keys={"a", "left"}, buttons={"dpleft"}, axis={"leftx", -0.5}}
bindings.right = {keys={"d", "right"}, buttons={"dpright"}, axis={"leftx", -0.5}}
bindings.interact = {keys={"e"}}
bindings.shop = {keys={"tab"}}
bindings.b = {keys={"j", "z"}, buttons={"a", "y"}}
bindings.a = {keys={"k", "x"}, buttons={"b", "x"}}
bindings.start = {keys={"escape"}, buttons={"start"}}
bindings.select = {keys={"space"}, buttons={"back"}}

local keyboardLut = {}
local joypadLut = {}
local pointerIsMouse = true

for k, v in pairs(bindings) do
    if v.keys then
        for i, key in ipairs(v.keys) do
            keyboardLut[key] = k
        end
    end

    if v.buttons then
        for i, button in ipairs(v.buttons) do
            joypadLut[button] = k
        end
    end
end

local input = {}
local state = {}
local previousState = {}

local function pressButton(btn)
    state[btn] = true
end

local function releaseButton(btn)
    state[btn] = false
end

function input.update(dt)
    for k, v in pairs(state) do
        previousState[k] = state[k]
    end
end

function input.discard()
    for k,v in pairs(state) do
        state[k] = false
        previousState[k] = false
    end
end

function input.isPressed(btn)
    return state[btn]
end

function input.justPressed(btn)
    return state[btn] and not previousState[btn] 
end

function input.justReleased(btn)
    return not state[btn] and previousState[btn]
end

function input.strength(btn)
    return input.isPressed(btn) and 1 or 0
end

function input.pointerPosition()
    return screenToWorld(love.mouse.getPosition())
end

function input.isPointerMouse()
    return pointerIsMouse
end

function love.keypressed(key, scancode)
    if keyboardLut[scancode] then
        pressButton(keyboardLut[scancode])
    else
        Signal.emit("keypressed", key, scancode)
    end
end

function love.keyreleased(key, scancode)
    if keyboardLut[scancode] then
        releaseButton(keyboardLut[scancode])
    else
        Signal.emit("keyreleased", key, scancode)
    end
end

function love.gamepadpressed(pad, button)
    if joypadLut[button] then
        pressButton(joypadLut[button])
    end
end

function love.gamepadreleased(pad, button)
    if joypadLut[button] then
        releaseButton(joypadLut[button])
    end
end

function love.mousepressed(x, y, button, touch)
    state["pointer"] = true
    pointerIsMouse = touch == false
end

function love.mousereleased(x, y, button, touch)
    state["pointer"] = false
    pointerIsMouse = touch == false
end

return input