
local level = {}

function level.loadMap(path)
    local m = love.filesystem.load(path)()
    local mapWidth, mapHeight = m.width, m.height
    local drawLayers = {}
    local levelSegments = {}
    local collisionMap = {}

    local function findSeg(x, y)
        for i, seg in ipairs(levelSegments) do
            if x >= seg.x and x < seg.x+seg.width and y >= seg.y and y < seg.y+seg.height then
                return seg
            end
        end
    end

    for i, layer in ipairs(m.layers) do
        if layer.type == "objectgroup" and layer.name == "segments" then
            for o, obj in ipairs(layer.objects) do 
                if obj.class == "segment" then 
                    table.insert(levelSegments, obj)
                end
            end
        elseif layer.type == "objectgroup" then
            for o, obj in ipairs(layer.objects) do
                local seg = findSeg(obj.x, obj.y)
                if not seg.objects then
                    seg.objects = {}
                end
                table.insert(seg.objects, obj)
            end
        
        elseif layer.type == "tilelayer" and layer.name == "collision" then 
            collisionMap = layer.data
        else
            if layer.type == "tilelayer" then
                table.insert(drawLayers, layer)
            end
            
        end
    end

    return {
        width = mapWidth, 
        height = mapHeight, 
        layers = drawLayers, 
        collisionMap = collisionMap, 
        segments = levelSegments
    }
end

function level.tileAt(x, y, map, layerId) 
    local tx, ty = math.floor(x/8), math.floor(y/8)
    local i = ty * map.width + tx

    local layer = map[layerId]
    
    return layer and layer[i + 1] or -1
end

return level