function math.round(x)
    return math.floor(x + 0.555)
end

function math.lerp(a, b, t)
    return a + (b - a) * t
end

function math.len(x, y)
    return math.sqrt(x*x + y*y)
end

function math.norm(x, y)
    local len = math.len(x, y)
    if len == 0 then
        return 0, 0
    end
    return x / len, y / len
end

function math.dist(x1, y1, x2, y2)
    return math.len(x2 - x1, y2 - y1)
end

function math.angle(x, y)
    return math.atan2(y, x)
end

function math.sign(x)
    if x < 0 then
        return -1
    elseif x > 0 then 
        return 1
    end
    return 0
end

function screenToWorld(x, y, dx, dy)
    dx = dx or 0
    dy = dy or 0
    local scale = math.min(love.graphics.getWidth() / WIDTH, love.graphics.getHeight() / HEIGHT)
    -- scale = math.floor(scale)

    local centreX, centreY = love.graphics.getWidth() / 2, love.graphics.getHeight() / 2
    local ox = (centreX - ((WIDTH * scale) / 2)) 
    local oy = (centreY - ((HEIGHT * scale) / 2))

    return ((x - ox) / scale) + dx , ((y - oy) / scale) + dy
end

-- some important colours used all over the place
colours = {
    white = {1, 1, 1, 1},
    mywhite = {love.math.colorFromBytes(253, 247, 237, 255)},
    black = {love.math.colorFromBytes(34, 34, 34, 255)},
    darkgrey = {0.2, 0.2, 0.25, 1},
    lightgrey = {0.8, 0.8, 0.8, 1},
    yellow = {love.math.colorFromBytes(250, 203, 62, 255)},
    red = {love.math.colorFromBytes(218, 78, 56, 255)},
    darkblue = {love.math.colorFromBytes(2, 86, 145, 255)},
    lightblue = {love.math.colorFromBytes(86, 152, 204, 255)},
}


D_WIDTH = 480
D_HEIGHT = 270
WIDTH = D_WIDTH
HEIGHT = D_HEIGHT

love.filesystem.setIdentity("lovejam24")
local windowScale = 2
font = love.graphics.newImageFont("assets/gfx/font.png", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz 0123456789!?%()[]\"'`/\\+-,$")
love.graphics.setFont(font)
love.graphics.setDefaultFilter("nearest", "nearest")
love.graphics.setLineWidth(1)
love.graphics.setLineStyle("rough")

local Signal = require("lib.hump.signal")
local drawing = require("drawing")
local assets = require("assets")
local input = require("input")
local tilemap = require("tilemap")
local state = require("state")
local music = require("music")


state.init({
    title = require("title"),
    game = require("game"),
    compmenu = require("compmenu"),
    shop = require("shop"),
    gameover = require("gameover"),
    placeentity = require("placeentity"),
    paused = require("paused")
})

local x, y = WIDTH / 2, HEIGHT / 2



local updateT,drawT = 0, 0

Signal.register("keyreleased", function(key, scan)
    if scan == "f11" and  love.system.getOS() ~= "Web" then
        love.graphics.captureScreenshot("screenshot.png")
    end
end)



local dtAcc = 0
local desiredTimestep = 1 / 60

state.switch("title")


function love.update(dt)
    dtAcc = dtAcc + dt

    music.update(dt)

    while dtAcc >= desiredTimestep do
        state.update(desiredTimestep)
        -- doupdate(desiredTimestep)
        input.update(dt)
        dtAcc = dtAcc - desiredTimestep
    end
end

function love.draw()
    drawing.start()

    state.draw()
    -- dodraw()
  
    drawing.stop()

    state.drawDebug(2, 40)
    
    drawing.present()
    -- dodrawdebug()

    -- love.window.setTitle(string.format("t: %.2fms, update: %.2fms, draw: %.2fms", updateT + drawT, updateT, drawT))
end
