local entity = require("entity")
local input = require("input")
local collision = require("collision")
local player = {}

local movementSpeed =  80
local t = 0


local function getInputDirection()
        local ix, iy = 0, 0
        ix = input.strength("right") - input.strength("left")
        iy = input.strength("down") - input.strength("up")
        return math.norm(ix, iy)
end


function player.make(x, y)
    local e = entity.make({x = x or 0, y = y or 0, cx = 0, cy = 2, cr = 4, dir = 1, hp = 5, maxHp = 5})
    entity.playAnimation(e, "elf_f_idle_anim")
    e.state = "normal"
    e.layer = "player"
    e.name = "player"
    e.update = player.update
    
    return e
end

function player.update(dt, player, gameData)
    t = t + dt
    if t > 99999 then
        t = 0
    end

    if player.state == "normal" then
        local stats = gameData.playerStats
        local ix, iy = getInputDirection()
        local mx = input.pointerPosition()
        
        local anim = "elf_m_idle_anim"

        if ix ~= 0 or iy ~= 0 then
            anim = "elf_m_run_anim"
        end

        player.fx = mx - WIDTH / 2 < 0

        entity.playAnimation(player, anim)
        
        local vx = ix * movementSpeed * stats.movementSpeedModifier * dt
        local vy = iy * movementSpeed * stats.movementSpeedModifier * dt
        local col = collision.moveObject(player, vx, vy, function(item, other)
            if other.layer == "enemies" or other.solid then
                return "slide"
            end
        end)
        if col and col.other.layer == "enemies" then
            player.state = "hit"
            player.vx = -ix * 60
            player.vy = -iy * 60
        end
    elseif player.state == "hit" then
        entity.playAnimation(player, "elf_m_hit_anim")
        if t > 0.016 *4 then
            t = 0
            player.visible = not player.visible
        end

        local col = collision.moveObject(player, player.vx * dt, player.vy * dt, function(item, other)
            if other.solid then
                return "slide"
            end
            return "cross"
        end)

        local friction = 0.9
        local obj = player
        obj.vx = obj.vx * friction
        obj.vy = obj.vy * friction 
        if math.len(obj.vx, obj.vy) < 2 then
            obj.vx = 0
            obj.vy = 0
            obj.state = "normal"
            player.visible = true
        end
    end
    
end



return player