local collision = require("collision")
local weapon = require("weapon")
local game = require("game")

local playerhelpers = {}

function playerhelpers.mouseAi(dt, mouse, gameData)
    local obj = mouse
    local items, count = collision.query(obj.x + obj.cx - obj.cr, obj.y + obj.cy - obj.cr, obj.cr * 2, obj.cr * 2, function(item)
        return item.layer == "collectable"
    end)

    for i = 1, count do
        local item = items[i]
        item.destroy = true
        gameData.currency = gameData.currency + (100 * gameData.reactor.efficiency)
    end

    local items, count = collision.query(0, 0, WIDTH, HEIGHT, function(item)
        return item.layer == "collectable" and not item.destroy
    end)

    local dx, dy = 0, 0
    if count > 0 then
        table.sort(items, function(l, r)
            local lDist = math.dist(l.x, l.y, mouse.x, mouse.y)
            local rDist = math.dist(r.x, r.y, mouse.x, mouse.y)
            return lDist <= rDist
        end)

        local closest = items[1]
        
        dx, dy = math.norm(closest.x - mouse.x ,closest.y - mouse.y)
    end

    collision.moveObject(mouse, dx * 64 * dt, dy * 64 * dt)

end

function playerhelpers.turretAi(dt, tur, gameData)
    tur.timer = tur.timer - dt
    if tur.timer <= 0 then
        
        
        local enemies = gameData.enemies
        
        local items, count = collision.query(tur.x - tur.radius, tur.y - tur.radius, tur.radius * 2, tur.radius * 2, function(item)
            return item.layer == "enemies"
        end)
        if count > 0 then
            table.sort(items, function(l, r) 
                local lDist = math.dist(l.x, l.y, tur.x, tur.y)
                local rDist = math.dist(r.x, r.y, tur.x, tur.y)
                return lDist <= rDist
            end)

            local sx, sy = tur.x, tur.y - 26
            
            local dx, dy = math.norm(items[1].x - sx, items[1].y - sy)
            game.addPlayerBullet({
                target = "enemies",
                vx = dx * 256, vy = dy * 256,
                sprite = "demo_bullet",
                cx = 0, cy = 0, cr = 2,
                update = weapon.updateProjectile,
                power = 1,
                projectile = true
            }, sx, sy)
            tur.timer = tur.cooldown
        end
    end
end

function playerhelpers.reset(_game) 
    game = game
    playerhelpers.weakWall = {
        layer = "player", 
        hp = 10, 
        maxHp = 10, 
        solid = true, 
        sprite = "column", 
        cx = 0, 
        cy = 0, 
        cr = 4
    }

    playerhelpers.turret = {
        layer = "player", 
        type = "turret",
        hp = 10, 
        maxHp = 10, 
        -- solid = true, 
        sprite = "turret", 
        cx = 0, 
        cy = -4, 
        cr = 4,
        radius = 32,
        cooldown = 2,
        timer = 2,
        update = playerhelpers.turretAi
    }

    playerhelpers.mouse = {
        layer = "player", 
        type = "mouse",
        hp = 5, 
        maxHp = 5, 
        -- solid = true, 
        animId = "goblin_run_anim",
        sprite = "goblin_run_anim_f0", 
        cx = 0, 
        cy = -4, 
        cr = 4,
      
        update = playerhelpers.mouseAi
    }
end

-- playerhelpers.reset()

return playerhelpers