local abilities = require("abilities")

local function add(amount) 
    return function(current)
        return current + amount
    end
end

local function subtract(amount) 
    return function(current)
        return current - amount
    end
end

local function multiply(amount)
    return function(current)
        return current * amount
    end
end

local data = {
    {
        sprite = "powerup_arrow",
        description = "+10% attack power",
        powerupType = "playerStats",
        stat = "attackPowerModifier",
        modify = add(0.1)
    }, 
    
    {
        sprite = "powerup_arrow",
        description = "1 more projectile",
        powerupType = "weapon",
        weapon = "arrow",
        stat = "count",
        modify = add(1)
    }, 
    
    {
        sprite = "powerup_arrow",
        description = "-5% attack cooldown",
        powerupType = "playerStats",
        stat = "attackCooldownModifier",
        modify = add(0.05) 
    }
}



for i = 1, #data do
    local powerup = data[i]
    if not powerup.apply then
        if powerup.powerupType == "playerStats" then
            powerup.apply = function(gameData)
                local stats = gameData.playerStats
                if stats[powerup.stat] then
                    stats[powerup.stat] = powerup.modify(stats[powerup.stat])
                end
            end
        elseif powerup.powerupType == "weapon" then
            powerup.apply = function(gameData)
                local a = abilities.getAbility(powerup.weapon)
                if a and a[powerup.stat] then
                    a[powerup.stat] = powerup.modify(a[powerup.stat])
                end
            end
        end
    end
end

local powerups = {}


function powerups.getPowerups(gameData)
    return {
        data[1],
        data[2],
        data[3]
    }
end

return powerups