local playerhelpers = require("playerhelpers")
local input = require("input")
local drawing = require("drawing")
local state = require("state")
local game = require("game")

local shop = {}

local tabs = {
    {
        text = "Items",
        x = 96, y = 32, w = 96, h = 16
    },
    {
        text = "Upgrades",
        x = 96+96, y = 32, w = 96, h = 16
    },
    {
        text = "Helpers",
        x = 96+96+96, y = 32, w = 96, h = 16
    },
}

local selectedTab = 1
local content

function shop.reset()
     content = {
    -- items
        {
            {
                text = "Small Medkit",
                cost = 40,
                apply = function()
                    local player = game.getPlayer()
                    player.hp = math.min(player.maxHp, player.hp + 3)
                end
            },

            {
                text = "Large Medkit",
                cost = 80,
                apply = function()
                    local player = game.getPlayer()
                    player.hp = player.maxHp
                end
            },

            {
                text = "Partial Reactor Repair",
                cost = 40,
                apply = function()
                    local player = game.getReactor()
                    player.hp = math.min(player.maxHp, player.hp + 6)
                end
            },

            {
                text = "Full Reactor Repair",
                cost = 120,
                apply = function()
                    local player = game.getReactor()
                    player.hp = player.maxHp
                end
            },


        },

        -- upgrades
        {
            {
                text = "Wall dur +",
                cost = 80,
                levels = 5,
                apply = function()
                    playerhelpers.weakWall.maxHp = playerhelpers.weakWall.maxHp + 3
                    playerhelpers.weakWall.hp = playerhelpers.weakWall.maxHp 
                end
            },
            {
                text = "Turret dur +",
                cost = 80,
                levels = 5,
                apply = function()
                    playerhelpers.turret.maxHp = playerhelpers.turret.maxHp + 3
                    playerhelpers.turret.hp = playerhelpers.turret.maxHp 
                end
            },
            {
                text = "Turret cooldown -",
                cost = 80,
                levels = 5,
                apply = function()
                    playerhelpers.turret.cooldown  = playerhelpers.turret.cooldown - 0.2
                    playerhelpers.turret.timer = playerhelpers.turret.cooldown
                end
            },
            {
                text = "Turret radius +",
                cost = 80,
                levels = 3,
                apply = function()
                    playerhelpers.turret.radius  = playerhelpers.turret.radius + 8
                    
                end
            },
            {
                text = "Mouse speed +",
                cost = 80,
                levels = 3,
                apply = function()
                
                end
            },
            {
                text = "Player Max HP +",
                cost = 80,
                levels = 7,
                apply = function()
                    local player = game.getPlayer()
                    player.maxHp = player.maxHp + 1
                end
            },
            {
                text = "Reactor efficiency +",
                cost = 200,
                levels = 3,
                apply = function()
                    local reactor = game.getReactor()
                    reactor.efficiency = reactor.efficiency + 1
                end
            },
            {
                text = "Reactor dur +",
                cost = 80,
                levels = 10,
                apply = function()
                    local reactor = game.getReactor()
                    reactor.maxHp = reactor.maxHp + 1
                end
            },
        },

        -- helpers
        {
            {
                
                cost = 35,
                prototype = "weakWall",
                text = "Wall",
                
            },

            {
                
                cost = 260,
                prototype = "turret",
                text = "Turret",
                
            },

            {
                
                cost = 260,
                prototype = "mouse",
                text = "Mouse",
                outText = "Already Owned"
            },
        }
    }
end

local optionHandlers = {
    -- items
    function(option)
        option.apply()
    end,

    -- upgrades
    function(option)
        if option.levels > 0 then
            option.apply()
            option.levels = option.levels - 1
            option.cost = option.cost * 2
        end
    end,

    --helpers
    function(option)
        state.pop()
        state.push("placeentity", playerhelpers[option.prototype])
    end
}

local function canAfford(option)
    return option.cost <= game.getCurrency()
end

local function isAvailable(option)
    if option.levels then
        return option.levels > 0 
    end

    if option.prototype == "mouse" then
        return not game.hasAMouse()
    end
    return true
end

local function mouseOverRect(x, y, w, h)
    local mx, my = input.pointerPosition()
    return  mx >= x and 
        mx <= x + w and
        my >= y and
        my <= y + h
end

function shop.init(fresh)
    if fresh then
        selectedTab = 1
    end
end

function shop.update(dt)
    if input.justPressed("start") then
        state.pop()
        state.push("compmenu")
    end

    if input.justPressed("pointer") then
        -- close button
        if mouseOverRect(368, 14, 16, 16) then
            state.pop()
            state.push("compmenu")
        end

        -- check tabs
        for i = 1, #tabs do
            local button = tabs[i]
            if mouseOverRect(button.x, button.y, button.w, button.h) then
                selectedTab = i
            end
        end

        -- check items
        local top = 32 + 24
        local left = 96 + 16
        local itemW =  64
        local itemH = 44

        local options = content[selectedTab]
        for i = 0, #options -1 do
            
            local option = options[i+1]
           
            local x = (i % 4) * itemW + left
            local y = math.floor(i / 4) * itemH + top

            if mouseOverRect(x, y, itemW, itemH) then
                if isAvailable(option) and canAfford(option) then
                    optionHandlers[selectedTab](option)
                    game.subtractCurrency(option.cost)
                end
            end
        end
    end
end

function shop.draw()
    drawing.col(0, 0, 0, 0.7)
    drawing.rect("fill", 0, 0, WIDTH, HEIGHT)
    drawing.col(1, 1, 1, 1)
    drawing.print(string.format("$%d", game.getCurrency()), 4, 30)
    drawing.sprite("uiClose", 368, 14)

    for i = 1, #tabs do
        local button = tabs[i]
        local spr = "tabUnselected"
        if i == selectedTab then
            spr = "tabSelected"
        end
        drawing.sprite(spr, button.x, button.y)
        drawing.printf(button.text, button.x, button.y + 3, button.w, "center")
    end

    drawing.sprite("uiPanel", 96, 32+16)

    local top = 32 + 24
    local left = 96 + 16
    local itemW =  64
    local itemH = 44

    local options = content[selectedTab]
    for i = 0, #options -1 do
        local option = options[i+1]
        local x = (i % 4) * itemW + left
        local y = math.floor(i / 4) * itemH + top

        drawing.col(1,1,1,1)
        if not isAvailable(option) or not canAfford(option) then
            drawing.col(0.7,0.7,0.7,1)
        end

        drawing.printf(option.text, x+1, y + 2, itemW-2, "center")
        drawing.printf(string.format("$%d", option.cost), x+2, y + itemH - 10 , itemW-4, "right")
        if option.levels then
            drawing.printf(tostring(option.levels), x+2, y + itemH - 10 , itemW-4, "left")
        end

        drawing.col(1,1,1,1)
        drawing.rect("line", x+1, y+1, itemW-2, itemH-2)

        
        if mouseOverRect(x, y, itemW, itemH) and isAvailable(option) and canAfford(option) then
            -- drawing.col
            drawing.rect("line", x, y, itemW, itemH)
        end

        if not isAvailable(option) then
            drawing.line(x, y, x + itemW, y + itemH)
            drawing.line(x, y + itemH, x + itemW, y)
            drawing.col(1,0,0,1)
            
            drawing.printf(option.outText or "Out of Stock", x+1, y + 16 , itemW-2, "center")
            drawing.col(1,1,1,1)
        end
    end

end

-- shop
return shop