local drawing = require("drawing")

local stack = {}

local state = {}

local states 


local function initCurrent(...)
    local current = state.current()
    if states[current].init then
        states[current].init(...)
    end    
end

local function exitCurrent()
    local current = state.current()
    if states[current].exit then
        states[current].exit()
    end  
end

local function pauseCurrent()
    local current = state.current()
    if current and states[current].pause then
        states[current].pause()
    end  
end

local function resumeCurrent()
    local current = state.current()
    if current and states[current].resume then
        states[current].resume()
    end  
end


function state.init(_states)
    states = _states
end

function state.current()
    return stack[#stack]
end



function state.push(name, ...)
    pauseCurrent()
    table.insert(stack, name)
    initCurrent(...)
end

function state.pop(name, reinit)
    reinit = reinit or false
    if #stack > 0 then
        table.remove(stack, #stack)
        exitCurrent()
        resumeCurrent()
        if reinit then
            initCurrent()
        end 
    end

end

function state.switch(name)
    stack = {}
    -- local current = state.current()
    -- if current ~= name then
    state.push(name)
    -- end
end

function state.update(dt)
    local s = states[state.current()]
    if s.update then
        s.update(dt)
    end
end

function state.draw()
    for i = 1, #stack do
        local s = states[stack[i]]
        if s.draw then
            s.draw()
        end
    end
end

function state.drawDebug(x, y)
    for i = 1, #stack do
        drawing.print(tostring(stack[i]), x, y + (i-1) * 10)
    end
end

return state