local input = require("input")
local abilities = require("abilities")
local drawing = require("drawing")

local powerupmenu = {}

local selected
local options
local acceptPointerInput 

local width = math.min(WIDTH - 2, 180-16-2)
local optionHeight = 42
local optionMargin = 2
local height = math.min(HEIGHT - 2, optionHeight * 3 + optionMargin * 2)
local pxPrev, pyPrev

local function getRectForOption(i)
    local x = math.floor((WIDTH - width) / 2 )
    local y = math.floor((HEIGHT - height) / 2) + (i * (optionHeight + optionMargin))
    return x, y, width, optionHeight
end

local function pointerOverOption(i, px, py) 
    local x, y, w, h = getRectForOption(i)
    return px >= x and px <= x + w and py >= y and py <= y + h
end


-- TODO pass in the things we're selecting from
function powerupmenu.init(powerups)
    acceptPointerInput = not input.isPressed("pointer")
    selected = 1
    options = powerups
    -- we want to start with fresh input state, which prevents player accidentally selecting an item when using mouse/touch
    input.discard()
end

local function optionSelected()
    if input.justReleased("pointer") then
        local px, py = input.pointerPosition()
        for i = 0, 2 do      
            if pointerOverOption(i, px, py) then
                selected = i + 1
                return true
            end
        end
    elseif input.justReleased("a") then
        return true
    end
end

function powerupmenu.update(dt, gameData)
    if input.justPressed("down") then
        selected = (selected % 3) + 1
    elseif input.justPressed("up") then
        if selected == 1 then 
            selected = 3
        else
            selected = selected - 1
        end
    end


    local px, py = input.pointerPosition()
    if px ~= pxPrev or py ~= pyPrev then
        for i = 0, 2 do      
            if pointerOverOption(i, px, py) and (input.isPointerMouse() or input.justPressed("pointer")) then
                selected = i + 1
                break
            end
        end
    end

       
    pxPrev, pyPrev = px, py
    
    if optionSelected() then
        print("selected, applying " .. tostring(selected))
        options[selected].apply(gameData)
        return true
    end
end

local function drawOptionBox(x, y, w, h, selected)
    
    drawing.col(colours.black)

    drawing.rect("fill", x+ 1, y + 1, w - 2, h -2)
    drawing.rect("fill",x, y, 5, 5)
    drawing.rect("fill",x + w - 5, y, 5, 5)

    drawing.rect("fill",x, y + h -5, 5, 5)
    drawing.rect("fill",x +w -5, y +h -5, 5, 5)

    drawing.col(colours.mywhite)
    if selected then
        drawing.col(colours.yellow)
    end

    
    drawing.rect("fill",x+1, y+1, 3, 3)
    drawing.rect("fill",x + w - 4, y+1, 3, 3)
    drawing.rect("fill",x+1, y+h-4, 3, 3)
    drawing.rect("fill",x + w - 4, y+h-4, 3, 3)
    drawing.line(x+1, y+3, x +w - 1, y+3)
    drawing.line(x+1, y+h-2, x +w - 1, y+h-2)
    drawing.line(x+3, y+3, x + 3, y+h -2)
    drawing.line(x+w-2, y+3, x + w-2, y+h -2)
    drawing.col(colours.white)
end
function powerupmenu.draw()
    drawing.col({0, 0, 0, 0.75})
    drawing.rect("fill", 0, 0, WIDTH, HEIGHT)
    local ty = math.floor((((HEIGHT - height) /2) - 11) /2) 
    ty = math.floor(((HEIGHT - height) /2) - 16) 

    drawing.col(colours.black)
    drawing.printf("Choose a Powerup!", -1, ty, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", 1, ty, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", 0, ty-1, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", 0, ty+1, WIDTH, "center" )

    drawing.printf("Choose a Powerup!", -1, ty-1, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", 1, ty-1, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", -1, ty+1, WIDTH, "center" )
    drawing.printf("Choose a Powerup!", 1, ty+1, WIDTH, "center" )

    drawing.col(colours.mywhite)
    drawing.printf("Choose a Powerup!", 0, ty, WIDTH, "center" )
    for i = 0, 2 do
        local option = options[i+1]
        local x, y, w, h = getRectForOption(i)
        drawOptionBox(x, y, w, h, selected and selected - 1 == i)

        drawing.sprite(option.sprite, x + 11, y + 11) 

        drawing.printf(option.description, x+11+20+4, math.floor(y+ ((42 - 9 )/ 2)), w - 11 - 11 - 20, "center")

    end
    drawing.col(colours.white)
end



return powerupmenu